{% for field in form.fields %}
<p class="form-row">
  {% if field.error %}
  <span class="error">{{ field.error }}</span>
  {% endif %}
  {% if field.label %}
  <label for="{{ field.id }}">{{ field.label }}</label>
  {% endif %}
  {{ field.render()|safe }}
</p>
{% endfor %}
