<table class="fieldset">
{% for subfield in fields %}
  {% if subfield.error %}
  <tr class="error-row">
    <td colspan="2">
      <span class="error">{{ subfield.error }}</span>
    </td>
  </tr>
  {% endif %}
  <tr class="form-row{% if loop.last %} last{% endif %}{% if loop.first %} first{% endif %}{% if subfield.classname %} {{ subfield.classname }}{% endif %}">
    <th>
      {% if subfield.label %}
      <label for="{{ subfield.id }}">{{ subfield.label }}</label>
      {% endif %}
    </th>
    <td>
      {{ subfield.render()|safe }}
    </td>
  </tr>
{% endfor %}
</table>
