<textarea id="{{ widget.id }}" name="{{ widget.input_name }}"
          {%- if readonly %} readonly="readonly"{% endif %}
          {%- if widget.classname %} class="{{ widget.classname }}"{% endif %}>
          {{- value|escape -}}
</textarea>
