{% for field in form.fields %}
  {% if field.error %}
  <tr class="error-row">
    <td colspan="2">
      <span class="error">{{ field.error }}</span>
    </td>
  </tr>
  {% endif %}
  <tr class="form-row{% if loop.last %} last{% endif %}{% if loop.first %} first{% endif %}{% if field.classname %} {{ field.classname }}{% endif %}">
    {% if field.label %}
      <th>
        <label for="{{ field.id }}">{{ field.label }}</label>
      </th>
    {% endif %}
    <td {% if not field.label %}colspan="2"{% endif %}>
      {{ field.render()|safe }}
    </td>
  </tr>
{% endfor %}
